// simple CPPM decoder for ATtiny85.  Outputs the first five channels to individual servos.  Set the ATtiny to use the internal 16MHz (or 16.5MHz) clock.
// ceptimus.  June 2021
// connections (these are the Digispark board references, but if you're using a bare chip, P0 is just bit 0 of PORTB, P1 is bit 1 and so on...)
// P0 CPPM input
// P1 servo output first channel
// P2 servo output second channel
// P3 servo output third channel
// P4 servo output fourth channel
// P5 (if it's not configured as a reset pin) servo output for fifth channel
// I'll do a very slightly modified version for the ATmega - that will use PORTD to get 8 servo outputs, but still keep the CPPM input pin on PB0.

void setup() {
  noInterrupts(); // disable all interrupts for the entire sketch
  PORTB = 0b00000000; // all servo outputs off
  DDRB  = 0b00111110; // five output pins for the (up to) five servos, one input pin for the incoming CPPM signal
  TCCR0A = 0x00;  // free-running timer/counter 0, counting up to 0xFF and wrapping back to zero
  TCCR0B = 0x05;  // with a 16MHz clock, the counter will increment every 64 us.  Longest 'normal' pulse should be about 32 counts.  Anything over 40 counts is a sync pulse
  uint8_t previousCount = TCNT0;  // initialize previousCount by getting current count from timer/counter 0
  while (true) { // loop forever
    asm( // inline assembly for fastest possible speed in 'waiting for edges' loops: sbrX takes up to 3 cycles, and rjmp takes 2 cycles: loops ~0.31 microseconds with 16MHz clock
      "loopA: sbic 0x16, 0 \n" // skip next instruction if bit 0 at I/O address 0x16 (PINB) is clear
      "rjmp loopA \n" // otherwise keep testing the bit - this waits for low state on CPPM input pin
      "loopB: sbis 0x16, 0 \n" // skip next instruction if bit 0 at I/O address 0x16 (PINB) is set
      "rjmp loopB \n" // otherwise keep testing the bit - this waits for high state on CPPM input pin
    );
    uint8_t currentCount = TCNT0;  // take current count from timer/counter 0
    if ((uint8_t)(currentCount - previousCount) > 40) PORTB = 0b00000010; // sync pulse detected - so switch first servo output on, and all others off
    else PORTB += PORTB; // else switch the next servo output on (and the previous one off)
    previousCount = currentCount; // the current count from this pass through the loop becomes the previous count for the next pass
  }
}

void loop() {}  // loop() not used because we have an endless loop inside setup();
